package com.unissoft.utils;

import cn.hutool.core.net.NetUtil;
import com.github.yitter.contract.IdGeneratorOptions;
import com.github.yitter.idgen.YitIdHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author qingsong.han
 * @description: 雪花漂移id-15 配置类
 * @create 2022-06-07 15:28
 */
@Slf4j
@Component
@Order(0)
@SuppressWarnings("ALL")
public class SystemInitConfig implements CommandLineRunner {

    @Resource
    private RedissonUtil redissonUtil;

    @Resource
    private RedisTemplate redisTemplate;

    @Value("${snowFlake.waitTime}")
    private long waitTime;

    @Value("${snowFlake.leaseTime}")
    private long leaseTime;

    /**
     * 分布式锁Key
     */
    private static final String CACHE_ID_GENERATOR = "LOCK_ID_GENERATOR";

    /**
     * 最大机器号Key
     */
    private static final String CACHE_WORKERID_MAXID = "CACHE_WORKERID_MAXID";

    /**
     * 已分配的机器号Key
     */
    private static final String CACHE_ID_IP = "CACHE_ID_IP";

    @Override
    public void run(String... args) throws Exception {
        //获取mac地址
        String macAddress = NetUtil.getLocalhost().getHostAddress();
        log.info("{} 配置分布式workerid 缓存========开始", macAddress);
        boolean existWorkerId = redisTemplate.opsForHash().hasKey(CACHE_ID_IP, macAddress);
        // 若已缓存在缓存中,直接跳过不设置
        if (existWorkerId) {
            Integer workerId = (Integer) redisTemplate.opsForHash().get(CACHE_ID_IP, macAddress);
            log.info("{} 已配置分布式workerid ...", macAddress);
            initWorkerId(workerId);
            return;
        }
        try {
            // 分布式锁等待120秒,执行时长最大120秒
            boolean locked = redissonUtil.tryLock(CACHE_ID_GENERATOR, waitTime, leaseTime);
            if (!locked) {
                throw new RuntimeException(macAddress + "设置分布式Id机器号失败");
            }
            ValueOperations<String, Integer> stringOperation = redisTemplate.opsForValue();
            // 先判断get,再set
            boolean initWorkerId = stringOperation.setIfAbsent(CACHE_WORKERID_MAXID, 1);
            if (!initWorkerId) {
                // 若已存在key,对最大的机器号自增1
                stringOperation.increment(CACHE_WORKERID_MAXID);
            }
            Integer workerId = stringOperation.get(CACHE_WORKERID_MAXID);
            initWorkerId(workerId);
            //设置mac地址 - workerid 到hash结构
            redisTemplate.opsForHash().put(CACHE_ID_IP, macAddress, workerId);
            log.info("已配置分布式workerid ,{} - {}", macAddress, workerId);
        } finally {
            redissonUtil.unlock(CACHE_ID_GENERATOR);
            log.info("{} 配置分布式workerid 缓存========结束", macAddress);
        }

    }

    private void initWorkerId(Integer workerId) {
        IdGeneratorOptions options = new IdGeneratorOptions(workerId.shortValue());
        YitIdHelper.setIdGenerator(options);
    }

    // 测试生成漂移雪花id
    /*public static void main(String[] args) {
        for (int i = 0; i < 10; i++) {
            System.out.println(YitIdHelper.nextId());
        }
    }*/

}
